@@ -4,9 +4,134 @@ from collections import Counter
 from ternary.helpers import simplex_iterator
 from matplotlib import pyplot as plt
 from matplotlib.patches import Patch
+from poisson_approval.meta_analysis.ternary_condorcet import draw_condorcet_zones
 
 
-def _generate_heatmap_data(f, scale, color_a, color_b, color_c):
+_WHITE = np.array([1., 1., 1.])
+_BRIGHTNESS = 0.5
+
+
+def _abc_to_rgb(abc):
+    """Conversion from candidates to RGB color
+
+    Parameters
+    ----------
+    abc : tuple, list or ndarray
+        Array of size 3. Each coefficient is associated to a candidate a, b, c. Typically the coefficients sum to 1
+        or less than 1.
+
+    Returns
+    -------
+    ndarray
+        Array of size 3. RGB color. Roughly speaking, candidate `a` is associated to red, `b` to green and `c` to blue,
+        but we try to mix colors more reasonably than just an average in RGB space.
+
+    Examples
+    --------
+        >>> _abc_to_rgb([1, 0, 0])
+        array([1. , 0.5, 0.5])
+        >>> _abc_to_rgb([.5, .5, 0])
+        array([1. , 1. , 0.5])
+        >>> _abc_to_rgb([0., 0., 0])
+        array([0.5, 0.5, 0.5])
+    """
+    norm_inf = np.max(abc)
+    if norm_inf == 0:
+        rgb = np.zeros(3)
+    else:
+        norm_1 = np.sum(abc)
+        rgb = np.array(abc) * np.min([norm_1, 1]) / norm_inf
+    return rgb * (1 - _BRIGHTNESS) + _WHITE * _BRIGHTNESS
+
+
+_COLOR_NO_ONE = _abc_to_rgb([0, 0, 0])
+_COLOR_A = _abc_to_rgb([1, 0, 0])
+_COLOR_B = _abc_to_rgb([0, 1, 0])
+_COLOR_C = _abc_to_rgb([0, 0, 1])
+_COLOR_AB = _abc_to_rgb([1, 1, 0])
+_COLOR_AC = _abc_to_rgb([1, 0, 1])
+_COLOR_BC = _abc_to_rgb([0, 1, 1])
+_COLOR_ABC = _abc_to_rgb([1, 1, 1])
+
+
+def _uni_mix(color_1, color_2, weight_1, weight_2):
+    """Mix two colors, with a fade that emphasizes the two base colors.
+
+    Parameters
+    ----------
+    color_1, color_2 : tuple, list or ndarray
+        Arrays of size 3 in format RGB.
+    weight_1, weight_2 : Number
+        Respective weights of the two colors.
+
+    Returns
+    -------
+    ndarray
+        Array of size 3 in format RGB. The mix is not linear. Currently, the resulting color is a barycenter
+        with respective weights ``weight_1**n`` and ``weight_2**n``, with ``n = 2.5``. This makes a "sharper"
+        transition, so that we see more of the base colors and less of the mixes.
+
+    Examples
+    --------
+        >>> import numpy as np
+        >>> _uni_mix(color_1=[1, 0, 0], color_2=[0, 1, 0], weight_1=1, weight_2=0)
+        array([1., 0., 0.])
+        >>> _uni_mix(color_1=[1, 0, 0], color_2=[0, 1, 0], weight_1=.5, weight_2=.5)
+        array([0.5, 0.5, 0. ])
+        >>> _uni_mix(color_1=[1, 0, 0], color_2=[0, 1, 0], weight_1=0, weight_2=1)
+        array([0., 1., 0.])
+    """
+    n = 2.5
+    return (np.array(color_1) * weight_1 ** n + np.array(color_2) * weight_2 ** n) / (weight_1 ** n + weight_2 ** n)
+
+
+def _palette(x, y):
+    """Compute the data for a nice color palette.
+
+    Parameters
+    ----------
+    x, y : Number
+        Cartesian coordinates of the point.
+
+    Returns
+    -------
+    ndarray
+        Array of size 3 in format RGB. The resulting figure consists of:
+
+        * A nice color palette in the unit disk,
+        * A black circle on the unit circle,
+        * White background outside.
+
+    Examples
+    --------
+        >>> _palette(x=0, y=1.5)
+        array([1., 1., 1.])
+        >>> _palette(x=0, y=0.99)
+        array([0., 0., 0.])
+        >>> color_almost_a = _palette(x=0, y=0.95)
+        >>> np.all(np.isclose(color_almost_a, [1, .5, .5], atol=1E-2))
+        True
+    """
+    z = x + 1j * y
+    r = np.abs(z)
+    if r > 1:
+        return np.array([1., 1., 1.])
+    if r > 0.96:
+        return np.array([0., 0., 0.])
+    theta = (np.angle(z, 'deg') + 30) % 360 - 30
+    for theta_sup, color_inf, color_sup in zip([30, 90, 150, 210, 270, 330],
+                                               [_COLOR_C, _COLOR_AC, _COLOR_A, _COLOR_AB, _COLOR_B, _COLOR_BC],
+                                               [_COLOR_AC, _COLOR_A, _COLOR_AB, _COLOR_B, _COLOR_BC, _COLOR_C]):
+        if theta <= theta_sup:
+            weight_inf = (theta_sup - theta) / 60
+            color = _uni_mix(color_sup, color_inf, 1 - weight_inf, weight_inf)
+            return _uni_mix(color, _COLOR_ABC, r ** .8, 1 - r ** .8)
+
+
+_DATA_PALETTE = np.array([[_palette(x / 100, y / 100) for x in range(-100, 100)] for y in range(-100, 100)])
+
+
+def _generate_heatmap_data(f, scale):
     """Generate RGBA data for a ``simplex to 3D'' heatmap plot.
 
     Parameters
@@ -16,12 +141,6 @@ def _generate_heatmap_data(f, scale, color_a, color_b, color_c):
         list of 3 numbers between 0 and 1.
     scale
         The scale of the ternary plot.
-    color_a
-        The RGB code representing candidate `a` (the first output). Each element of the tuple must be in [0, 1].
-    color_b
-        The RGB code representing candidate `b` (the second output). Each element of the tuple must be in [0, 1].
-    color_c
-        The RGB code representing candidate `c` (the third output). Each element of the tuple must be in [0, 1].
 
     Returns
     -------
@@ -33,20 +152,48 @@ def _generate_heatmap_data(f, scale, color_a, color_b, color_c):
     --------
         >>> def f(right, top, left):
         ...     return [right, 0, 0]
-        >>> _generate_heatmap_data(f, scale=2, color_a=(1., 0., 0.), color_b=(0., 1., 0.), color_c=(0., 0., 1.))
-        {(0, 0, 2): (0.0, 0.0, 0.0, 1.0), (0, 1, 1): (0.0, 0.0, 0.0, 1.0), (0, 2, 0): (0.0, 0.0, 0.0, 1.0), \
-(1, 0, 1): (0.5, 0.0, 0.0, 1.0), (1, 1, 0): (0.5, 0.0, 0.0, 1.0), (2, 0, 0): (1.0, 0.0, 0.0, 1.0)}
+        >>> _generate_heatmap_data(f, scale=2)
+        {(0, 0, 2): (0.5, 0.5, 0.5, 1.0), (0, 1, 1): (0.5, 0.5, 0.5, 1.0), (0, 2, 0): (0.5, 0.5, 0.5, 1.0), \
+(1, 0, 1): (0.75, 0.5, 0.5, 1.0), (1, 1, 0): (0.75, 0.5, 0.5, 1.0), (2, 0, 0): (1.0, 0.5, 0.5, 1.0)}
     """
     d = dict()
     for (right, top, left) in simplex_iterator(scale):
         values = f(right / scale, top / scale, left / scale)
-        color = (values[0] * np.array(color_a)
-                 + values[1] * np.array(color_b)
-                 + values[2] * np.array(color_c))
+        color = _abc_to_rgb(values)
         d[(right, top, left)] = (float(color[0]), float(color[1]), float(color[2]), 1.)
     return d
 
 
+def ternary_figure(size_inches='auto', scale=None, boundary_width=1.0, **kwargs):  # pragma: no cover
+    """Create a ternary plot (adaptation of `figure` from the package python-ternary).
+
+    Parameters
+    ----------
+    size_inches : tuple or str
+        The horizontal and vertical sizes of the figure, in inches. If 'auto', we will try to do our best.
+    scale : Number
+        The scale of the figure. The higher it is, the higher the resolution of the heatmap for example.
+    boundary_width
+        Width of the line representing the boundary of the triangle.
+    kwargs
+        Other keyword arguments are passed to the function `figure` of the package python-ternary.
+
+    Returns
+    -------
+    figure : matplotlib.figure.Figure
+    ternary_ax : TernaryAxesSubplotPoisson
+    """
+    ternary_ax = TernaryAxesSubplotPoisson(scale=scale, size_inches=size_inches, **kwargs)
+    figure = ternary_ax.get_figure()
+    if size_inches == 'auto':
+        figure.set_size_inches(5, 5)
+    else:
+        figure.set_size_inches(*size_inches)
+    plt.axis('off')
+    ternary_ax.boundary(linewidth=boundary_width)
+    return figure, ternary_ax
+
+
 class TernaryAxesSubplotPoisson(ternary.TernaryAxesSubplot):  # pragma: no cover
     """Subclass of `TernaryAxesSubplot`, defined in the package python-ternary.
 
@@ -207,9 +354,7 @@ class TernaryAxesSubplotPoisson(ternary.TernaryAxesSubplot):  # pragma: no cover
             plt.gcf().set_size_inches(7, 5)
 
     def heatmap_candidates(self, func, right_label, top_label, left_label, legend_title='',
-                           color_a=(1, 0.5, 0.5), color_b=(0.5, 1, 0.5), color_c=(0.5, 0.5, 1),
-                           color_a_edge=(.6, 0, 0), color_b_edge=(0, .6, 0), color_c_edge=(0, 0, .6),
-                           style='hexagonal', colorbar=False, **kwargs):
+                           legend_style='palette', style='hexagonal', colorbar=False, **kwargs):
         """Heatmap of a function from the simplex to 3D vectors.
 
         Parameters
@@ -225,18 +370,9 @@ class TernaryAxesSubplotPoisson(ternary.TernaryAxesSubplot):  # pragma: no cover
             Label of the left corner.
         legend_title : str
             Title of the legend.
-        color_a
-            RGB color associated to candidate `a` (i.e. first output of the function). Default: red.
-        color_b
-            RGB color associated to candidate `b` (i.e. second output of the function). Default: green.
-        color_c
-            RGB color associated to candidate `c` (i.e. third output of the function). Default: blue.
-        color_a_edge
-            RGB color used for the edge of the patch for candidate `a` in the legend.
-        color_b_edge
-            RGB color used for the edge of the patch for candidate `b` in the legend.
-        color_c_edge
-            RGB color used for the edge of the patch for candidate `c` in the legend.
+        legend_style : str
+            The style of the legend. The two available options are ``'palette'`` and ``'color_patches'``.
+            Cf. :meth:`legend_palette` and :meth:`legend_color_patches`.
         style
             Contrarily to default settings in python-ternary, the default is ``'hexagonal'``.
         colorbar
@@ -244,34 +380,54 @@ class TernaryAxesSubplotPoisson(ternary.TernaryAxesSubplot):  # pragma: no cover
         kwargs
             All other keywords arguments are passed to method ``heatmap`` of python-ternary.
         """
-        data = _generate_heatmap_data(func, self.get_scale(), color_a, color_b, color_c)
+        data = _generate_heatmap_data(func, self.get_scale())
         self.heatmap(data, style=style, colorbar=colorbar, use_rgba=True, **kwargs)
         self.right_corner_label(right_label)
         self.top_corner_label(top_label)
         self.left_corner_label(left_label)
         if self.size_inches == 'auto':
             plt.gcf().set_size_inches(5, 5)
-        legend_elements = [Patch(facecolor=color_a, edgecolor=color_a_edge, label='$a$'),
-                           Patch(facecolor=color_b, edgecolor=color_b_edge, label='$b$'),
-                           Patch(facecolor=color_c, edgecolor=color_c_edge, label='$c$')]
-        self.legend(title=legend_title, handles=legend_elements)
+        if legend_style == 'palette':
+            self.legend_palette(title=legend_title)
+        else:
+            self.legend_color_patches(title=legend_title, data=data)
 
-    def annotate_condorcet(self, right_ranking, top_ranking, left_ranking):
+    def annotate_condorcet(self, right_order, top_order, left_order, d_order_fixed_share=None):
         """Annotate who is the Condorcet winner depending on the region.
 
-        This method can be used when each point of the simplex represents a profile with only three different types,
-        each coordinate representing the share of one type. It annotates the regions according to which candidate
-        is the Condorcet winner, and indicates where no one is the Condorcet winner..
+        We consider a simplex where:
+
+        * Fixed shares of voters have preference orders given by `d_order_fixed_share`.
+        * The remaining voters are split between `right_order`, `top_order` and `left_order` in proportions that are
+          given by the point in the simplex.
+
+        This method annotates the regions according to which candidate is the Condorcet winner, and indicates where no
+        one is the Condorcet winner.
+
+        If there are weak orders and/or fixed shares of voters, the method may not work on all distributions because it
+        relies on an external package called `shapely`. If there are rankings only and no fixed shares of voters, it is
+        supposed to work on all distributions.
 
         Parameters
         ----------
-        right_ranking : str
-            The ranking whose share is maximal at the right corner.
-        top_ranking : str
-            The ranking whose share is maximal at the top corner.
-        left_ranking : str
-            The ranking whose share is maximal at the left corner.
+        right_order : str
+            The order whose share is maximal at the right corner.
+        top_order : str
+            The order whose share is maximal at the top corner.
+        left_order : str
+            The order whose share is maximal at the left corner.
+        d_order_fixed_share : dict, optional
+            Key: order. Value: a fixed share of voters in [0, 1].
         """
+        try:
+            draw_condorcet_zones(self, right_order, top_order, left_order, d_order_fixed_share)
+        except NameError:
+            self._annotate_condorcet_old(right_order, top_order, left_order, d_order_fixed_share)
+
+    def _annotate_condorcet_old(self, right_ranking, top_ranking, left_ranking, d_order_fixed_share):
+        """Old version, for rankings only"""
+        if d_order_fixed_share is not None:
+            raise NotImplementedError
         count_candidate_tops = dict(Counter([
             ranking[0] for ranking in [right_ranking, top_ranking, left_ranking]]))
         if len(count_candidate_tops) == 1:
@@ -332,32 +488,68 @@ class TernaryAxesSubplotPoisson(ternary.TernaryAxesSubplot):  # pragma: no cover
                     self.annotate_simplex('%s is CW' % top_ranking[0], (0.15, 0.7, 0.15))
                     self.annotate_simplex('%s is CW' % left_ranking[0], (0.25, 0.25, 0.5))
 
+    def legend_color_patches(self, title, all_mixes=False, data=None):
+        """Add a "color patches" legend to the current figure (for candidates heat maps).
 
-def ternary_figure(size_inches='auto', scale=None, boundary_width=1.0, **kwargs):  # pragma: no cover
-    """Create a ternary plot (adaptation of `figure` from the package python-ternary).
+        Parameters
+        ----------
+        title : str
+            Title of the legend.
+        all_mixes : bool
+            If True, then all mixes are indicated in the legend: a + b, a + c, b + c and a + b + c.
+        data : dict, optional
+            Key : anything. Value: a tuple, list, etc giving an RGB or RGBA code. This is used only if `all_mixes`
+            is False.
 
-    Parameters
-    ----------
-    size_inches : tuple or str
-        The horizontal and vertical sizes of the figure, in inches. If 'auto', we will try to do our best.
-    scale : Number
-        The scale of the figure. The higher it is, the higher the resolution of the heatmap for example.
-    boundary_width
-        Width of the line representing the boundary of the triangle.
-    kwargs
-        Other keyword arguments are passed to the function `figure` of the package python-ternary.
+        Notes
+        -----
+        * The color patches for single candidates a, b, c are in the legend anyway.
+        * If `all_mixes` is True, then all mixes are indicated in the legend.
+        * If `all_mixes` is False and `data` is given, then if at least a value is close to a perfect mix (like half
+          of `a` and half of `b`), then this mix is added to the legend.
+        """
+        def _edge_color(color):
+            return [max(x - .4, 0) for x in color]
+        legend_elements = [Patch(facecolor=_COLOR_A, edgecolor=_edge_color(_COLOR_A), label='$a$'),
+                           Patch(facecolor=_COLOR_B, edgecolor=_edge_color(_COLOR_B), label='$b$'),
+                           Patch(facecolor=_COLOR_C, edgecolor=_edge_color(_COLOR_C), label='$c$')]
+        if all_mixes or data is not None:
+            def add_optional_patch(color, label, insertion=False):
+                if all_mixes or (data is not None
+                                 and any(np.all(np.isclose(data_col[0:3], color, atol=1E-1))
+                                         for data_col in data.values())):
+                    patch = Patch(facecolor=color, edgecolor=_edge_color(color), label=label)
+                    if insertion:
+                        legend_elements.insert(0, patch)
+                    else:
+                        legend_elements.append(patch)
+            add_optional_patch(_COLOR_NO_ONE, 'No one', insertion=True)
+            add_optional_patch(_COLOR_AB, 'a, b')
+            add_optional_patch(_COLOR_AC, 'a, c')
+            add_optional_patch(_COLOR_BC, 'b, c')
+            add_optional_patch(_COLOR_ABC, 'a, b, c')
+        self.legend(title=title, handles=legend_elements)
 
-    Returns
-    -------
-    figure : matplotlib.figure.Figure
-    ternary_ax : TernaryAxesSubplotPoisson
-    """
-    ternary_ax = TernaryAxesSubplotPoisson(scale=scale, size_inches=size_inches, **kwargs)
-    figure = ternary_ax.get_figure()
-    if size_inches == 'auto':
-        figure.set_size_inches(5, 5)
-    else:
-        figure.set_size_inches(*size_inches)
-    plt.axis('off')
-    ternary_ax.boundary(linewidth=boundary_width)
-    return figure, ternary_ax
+    @staticmethod
+    def legend_palette(title):
+        """Add a "palette" legend to the current figure (for candidates heat maps).
+
+        Parameters
+        ----------
+        title : str
+            Title of the legend.
+
+        Notes
+        -----
+        This is rather a hack than a real matplotlib legend. Hence the usual commands to choose the position
+        of the legend, for example, are not meant to work here.
+        """
+        legend_ax = plt.gcf().add_axes([.68, .63, .2, .2], facecolor='k')
+        legend_ax.imshow(_DATA_PALETTE, origin='lower', interpolation='gaussian')
+        legend_ax.set_xticks([])
+        legend_ax.set_yticks([])
+        legend_ax.axis('off')
+        legend_ax.annotate('a', (100, 169), horizontalalignment='center', verticalalignment='center')
+        legend_ax.annotate('b', (35, 63), horizontalalignment='center', verticalalignment='center')
+        legend_ax.annotate('c', (165, 63), horizontalalignment='center', verticalalignment='center')
+        legend_ax.set_title(title)
