@@ -18,7 +18,8 @@ from poisson_approval.events.EventPivotWeak import EventPivotWeak
 from poisson_approval.utils.computation_engine import computation_engine
 from poisson_approval.utils.DictPrintingInOrder import DictPrintingInOrder
 from poisson_approval.utils.DictPrintingInOrderIgnoringZeros import DictPrintingInOrderIgnoringZeros
-from poisson_approval.utils.Util import sort_ballot, my_division
+from poisson_approval.utils.Util import my_division
+from poisson_approval.utils.UtilBallots import sort_ballot
 from poisson_approval.utils.UtilCache import cached_property
 
 
@@ -178,7 +179,7 @@ phi_ab = 0.707107>
         # Normalize if necessary
         total = sum(self.d_ballot_share.values())
         if not self.ce.look_equal(total, 1):
-            if normalization_warning:
+            if normalization_warning and not self.ce.look_equal(total, 1, rel_tol=1e-5):
                 warnings.warn(NORMALIZATION_WARNING)
             for ballot in self.d_ballot_share.keys():
                 self.d_ballot_share[ballot] = my_division(self.d_ballot_share[ballot], total)
