@@ -45,13 +45,13 @@ def rand_simplex(d=6):
     return np.concatenate((x, [1])) - np.concatenate(([0], x))
 
 
-def rand_integers_fixed_sum(n_integers, fixed_sum):
+def rand_integers_fixed_sum(d, fixed_sum):
     """Generate integers with a given sum (uniformly).
 
     Parameters
     ----------
-    n_integers : int
-        The desired number of integers.
+    d : int
+        The desired number of integers. In other words, we consider a simplex of dimension `d - 1`.
     fixed_sum : int
         The fixed sum.
 
@@ -63,10 +63,10 @@ def rand_integers_fixed_sum(n_integers, fixed_sum):
     Examples
     --------
         >>> initialize_random_seeds()
-        >>> rand_integers_fixed_sum(n_integers=6, fixed_sum=100)
+        >>> rand_integers_fixed_sum(d=6, fixed_sum=100)
         array([ 2, 23, 34,  0, 22, 19])
     """
-    n_separators = n_integers - 1
+    n_separators = d - 1
     separators = np.concatenate((
         [-1],
         np.sort(np.random.choice(fixed_sum + n_separators, n_separators, replace=False)),
@@ -96,68 +96,69 @@ def rand_simplex_grid(d, denominator):
         array([Fraction(13, 50), Fraction(13, 20), Fraction(9, 100)], dtype=object)
     """
     return np.array([
-        Fraction(int(n), denominator)
-        for n in rand_integers_fixed_sum(n_integers=d, fixed_sum=denominator)])
+        my_division(int(n), denominator)
+        for n in rand_integers_fixed_sum(d=d, fixed_sum=denominator)])
 
 
-def probability(generator, n_samples, test, conditional_on=None):
+def probability(factory, n_samples, test, conditional_on=None):
     """Probability that a random `something` meets some given test.
 
     Parameters
     ----------
-    generator : callable or tuple
+    factory : callable or tuple
         This can be:
 
         * Either a callable that takes no input and that outputs a (random) `something`,
-        * Or a tuple of such generators (cf. examples below).
+        * Or a tuple of such factories (cf. examples below).
     n_samples : int
         Number of samples.
     test : callable
-        A function that take as input(s) the output(s) of the generator(s) and that returns a Boolean.
+        A function that take as input(s) the output(s) of the factory(ies) and that returns a Boolean.
     conditional_on : callable
-        A function that take as input(s) the output(s) of the generator(s) and that returns a Boolean.
+        A function that take as input(s) the output(s) of the factory(ies) and that returns a Boolean.
         Default: always True.
 
     Returns
     -------
     float
-        The probability that the output(s) generated by `generator` meet(s) `test`, conditional on the fact
+        The probability that the output(s) generated by `factory` meet(s) `test`, conditional on the fact
         that it meets `conditional_on`, based on a Monte-Carlo estimation of `n_samples` trials.
 
     Examples
     --------
-    In this basic example with one generator, we estimate the probability that a random float between 0 and 1 is greater
+    In this basic example with one factory, we estimate the probability that a random float between 0 and 1 is greater
     than .5, conditionally on being greater than .25:
 
         >>> initialize_random_seeds()
-        >>> def generator():
+        >>> def rand_number():
         ...     return random.random()
-        >>> probability(generator, n_samples=1000, test=lambda x: x > .5, conditional_on=lambda x: x > .25)
+        >>> probability(factory=rand_number, n_samples=1000,
+        ...             test=lambda x: x > .5, conditional_on=lambda x: x > .25)
         0.661
 
-    In this example with a tuple of generators, we estimate the probability that a random 2*2 matrix and a random vector
+    In this example with a tuple of factories, we estimate the probability that a random 2*2 matrix and a random vector
     of size 2, both with integer coefficients between -10 included and 11 excluded, have a dot product that is null,
     conditionally on not being null themselves:
 
         >>> initialize_random_seeds()
-        >>> def generator_matrix():
+        >>> def rand_matrix():
         ...     return np.random.randint(-10, 11, (2, 2))
-        >>> def generator_vector():
+        >>> def rand_vector():
         ...     return np.random.randint(-10, 11, 2)
         >>> def test_dot_zero(matrix, vector):
         ...     return np.all(np.dot(matrix, vector) == 0)
         >>> def test_non_trivial(matrix, vector):
         ...     return not np.all(matrix == 0) and not np.all(vector == 0)
-        >>> probability(generator=(generator_matrix, generator_vector), n_samples=10000,
+        >>> probability(factory=(rand_matrix, rand_vector), n_samples=10000,
         ...             test=test_dot_zero, conditional_on=test_non_trivial)
         0.0003
     """
-    if not isinstance(generator, tuple):
-        generator = (generator, )
+    if not isinstance(factory, tuple):
+        factory = (factory,)
     success = 0
     i_samples = 0
     while i_samples < n_samples:
-        somethings = [g() for g in generator]
+        somethings = [g() for g in factory]
         if conditional_on is None or conditional_on(*somethings):
             i_samples += 1
             if test(*somethings):
@@ -165,67 +166,67 @@ def probability(generator, n_samples, test, conditional_on=None):
     return success / n_samples
 
 
-def image_distribution(generator, n_samples, f, conditional_on=None):
+def image_distribution(factory, n_samples, f, conditional_on=None):
     """Distribution of ``f(something)`` for a random `something`.
 
     Parameters
     ----------
-    generator : callable or tuple
+    factory : callable or tuple
         This can be:
 
         * Either a callable that takes no input and that outputs a (random) `something`,
-        * Or a tuple of such generators (cf. examples below).
+        * Or a tuple of such factories (cf. examples below).
     n_samples : int
         Number of samples.
     f : callable
-        A function that take as input(s) the output(s) of the generator(s)..
+        A function that take as input(s) the output(s) of the factory(ies).
     conditional_on : callable
-        A function that take as input(s) the output(s) of the generator(s) and that returns a Boolean.
+        A function that take as input(s) the output(s) of the factory(ies) and that returns a Boolean.
         Default: always True.
 
     Returns
     -------
     DictPrintingInOrder
         Keys: the obtained outputs for `f`. Values: the probability that `f(something)` has this output when `something`
-        is generated by `generator`, conditional on the fact that it meets `conditional_on`,
+        is generated by `factory`, conditional on the fact that it meets `conditional_on`,
         based on a Monte-Carlo estimation of `n_samples` trials.
 
     Examples
     --------
-    In this basic example with one generator, we compute the distribution of `n` modulo 10, when `n` is drawn uniformly
+    In this basic example with one factory, we compute the distribution of `n` modulo 10, when `n` is drawn uniformly
     at random between 0 included and 100 excluded, conditionally on the fact that `n` is even:
 
         >>> initialize_random_seeds()
-        >>> def generator_integer():
+        >>> def rand_integer():
         ...     return np.random.randint(0, 100)
         >>> def modulo_10(n):
         ...     return n % 10
         >>> def is_even(n):
         ...     return n % 2 == 0
-        >>> image_distribution(generator=generator_integer, n_samples=100, f=modulo_10, conditional_on=is_even)
+        >>> image_distribution(factory=rand_integer, n_samples=100, f=modulo_10, conditional_on=is_even)
         {0: 0.21, 2: 0.21, 4: 0.2, 6: 0.18, 8: 0.2}
 
-    In this example with a tuple of generator, we compute the distribution of `a mod b`, when `a` is drawn uniformly
+    In this example with a tuple of factories, we compute the distribution of `a mod b`, when `a` is drawn uniformly
     at random between 0 included and 100 excluded, and `b` is drawn uniformly at random between 1 included and 11
     excluded:
 
         >>> initialize_random_seeds()
-        >>> def generator_integer():
+        >>> def rand_integer():
         ...     return np.random.randint(0, 100)
-        >>> def generator_divider():
+        >>> def rand_divider():
         ...     return np.random.randint(1, 11)
         >>> def modulo(a, b):
         ...     return a % b
-        >>> image_distribution(generator=(generator_integer, generator_divider),
+        >>> image_distribution(factory=(rand_integer, rand_divider),
         ...                    n_samples=100, f=modulo)
         {0: 0.31, 1: 0.16, 2: 0.18, 3: 0.12, 4: 0.07, 5: 0.04, 6: 0.02, 7: 0.08, 9: 0.02}
     """
-    if not isinstance(generator, tuple):
-        generator = (generator, )
+    if not isinstance(factory, tuple):
+        factory = (factory,)
     d_result_occurrences = dict()
     i_samples = 0
     while i_samples < n_samples:
-        somethings = [g() for g in generator]
+        somethings = [g() for g in factory]
         if conditional_on is None or conditional_on(*somethings):
             i_samples += 1
             result = f(*somethings)
@@ -340,185 +341,6 @@ def isneginf(x):
         return np.isneginf(x)
 
 
-def sort_ballot(ballot):
-    """Put a ballot in alphabetical order.
-
-    Parameters
-    ----------
-    ballot : str
-        A ballot, e.g. ``'a'``, ``'ab'``, ``'ba'``, etc.
-
-    Returns
-    -------
-    str
-        The same ballot in alphabetical order.
-
-    Examples
-    --------
-        >>> sort_ballot('a')
-        'a'
-        >>> sort_ballot('ab')
-        'ab'
-        >>> sort_ballot('ba')
-        'ab'
-    """
-    return ''.join(sorted(ballot))
-
-
-def ballot_one(ranking):
-    """Ballot for the voter's preferred candidate.
-
-    Parameters
-    ----------
-    ranking : str
-        A ranking.
-
-    Returns
-    -------
-    str
-        The first candidate.
-
-    Examples
-    --------
-        >>> ballot_one('abc')
-        'a'
-    """
-    return ranking[0]
-
-
-def ballot_two(ranking):
-    """Ballot for the voter's second most-liked candidate.
-
-    Parameters
-    ----------
-    ranking : str
-        A ranking.
-
-    Returns
-    -------
-    str
-        The second candidate.
-
-    Examples
-    --------
-        >>> ballot_two('abc')
-        'b'
-    """
-    return ranking[1]
-
-
-def ballot_one_two(ranking):
-    """Ballot for the voter's two preferred candidates.
-
-    Parameters
-    ----------
-    ranking : str
-        A ranking.
-
-    Returns
-    -------
-    str
-        The ballot with the two first candidates.
-
-    Examples
-    --------
-        >>> ballot_one_two('abc')
-        'ab'
-        >>> ballot_one_two('bac')
-        'ab'
-    """
-    return sort_ballot(ranking[:2])
-
-
-def ballot_one_three(ranking):
-    """Ballot for the voter's first and third candidates.
-
-    Parameters
-    ----------
-    ranking : str
-        A ranking.
-
-    Returns
-    -------
-    str
-        The ballot with the first and third candidates.
-
-    Examples
-    --------
-        >>> ballot_one_three('abc')
-        'ac'
-        >>> ballot_one_three('cba')
-        'ac'
-    """
-    return sort_ballot(ranking[0] + ranking[2])
-
-
-def ballot_low_u(ranking, voting_rule):
-    """Ballot chosen by the voters who have a low utility for their middle candidate.
-
-    Parameters
-    ----------
-    ranking : str
-        A ranking.
-    voting_rule : str
-        The voting rule. Possible values are ``APPROVAL``, ``PLURALITY`` and ``ANTI_PLURALITY``.
-
-    Returns
-    -------
-    str
-        The ballot chosen by the voters with this ranking and a low utility for their middle candidate, in case
-        the response is utility-dependent.
-
-    Examples
-    --------
-        >>> ballot_low_u('abc', APPROVAL)
-        'a'
-        >>> ballot_low_u('abc', PLURALITY)
-        'a'
-        >>> ballot_low_u('abc', ANTI_PLURALITY)
-        'ac'
-    """
-    if voting_rule in {APPROVAL, PLURALITY}:
-        return ballot_one(ranking)
-    elif voting_rule == ANTI_PLURALITY:
-        return ballot_one_three(ranking)
-    else:
-        raise NotImplementedError
-
-
-def ballot_high_u(ranking, voting_rule):
-    """Ballot chosen by the voters who have a high utility for their middle candidate.
-
-    Parameters
-    ----------
-    ranking : str
-        A ranking.
-    voting_rule : str
-        The voting rule. Possible values are ``APPROVAL``, ``PLURALITY`` and ``ANTI_PLURALITY``.
-
-    Returns
-    -------
-    str
-        The ballot chosen by the voters with this ranking and a high utility for their middle candidate, in case
-        the response is utility-dependent.
-
-    Examples
-    --------
-        >>> ballot_high_u('abc', APPROVAL)
-        'ab'
-        >>> ballot_high_u('abc', PLURALITY)
-        'b'
-        >>> ballot_high_u('abc', ANTI_PLURALITY)
-        'ab'
-    """
-    if voting_rule in {APPROVAL, ANTI_PLURALITY}:
-        return ballot_one_two(ranking)
-    elif voting_rule == PLURALITY:
-        return ballot_two(ranking)
-    else:
-        raise NotImplementedError
-
-
 def give_figure(n, singular, plural=None):
     """Combine a number with a unit, whose word can be singular or plural.
 
@@ -808,98 +630,6 @@ def one_over_log_log_t_plus_fifteen(t):
     return my_division(1, math.log(math.log(t + 15)))
 
 
-def is_weak_order(o):
-    """Whether an object is a weak order.
-
-    Parameters
-    ----------
-    o : object
-
-    Returns
-    -------
-    bool
-        True iff `o` is a string that represent a weak order, i.e. of the form 'a>b~c' (lover) or 'a~b>c' (hater).
-
-    Examples
-    --------
-        >>> is_weak_order('a>b~c')
-        True
-        >>> is_weak_order('a>b~')
-        False
-        >>> is_weak_order(42)
-        False
-    """
-    return isinstance(o, str) and len(o) == 5 and {o[1], o[3]} == {'>', '~'}
-
-
-def is_lover(weak_order):
-    """Whether a weak order represents a "lover".
-
-    Parameters
-    ----------
-    weak_order : str
-        A weak order, e.g. ``'a>b~c'``, ``'a~b>c'``, etc.
-
-    Returns
-    -------
-    bool
-        True iff the weak order is of the form ``'a>b~c'``.
-
-    Examples
-    --------
-        >>> is_lover('a>b~c')
-        True
-    """
-    return weak_order[1] == '>'
-
-
-def is_hater(weak_order):
-    """Whether a weak order represents a "hater".
-
-    Parameters
-    ----------
-    weak_order : str
-        A weak order, e.g. ``'a>b~c'``, ``'a~b>c'``, etc.
-
-    Returns
-    -------
-    bool
-        True iff the weak order is of the form ``'a~b>c'``.
-
-    Examples
-    --------
-        >>> is_hater('a~b>c')
-        True
-    """
-    return weak_order[3] == '>'
-
-
-def sort_weak_order(weak_order):
-    """Put a weak order in normalized format (with the indifferent candidates sorted alphabetically).
-
-    Parameters
-    ----------
-    weak_order : str
-        A weak order, e.g. ``'a>b~c'``, ``'a~b>c'``, etc.
-
-    Returns
-    -------
-    str
-        The same weak order in normalized format.
-
-    Examples
-    --------
-        >>> sort_weak_order('a>c~b')
-        'a>b~c'
-        >>> sort_weak_order('b~a>c')
-        'a~b>c'
-    """
-    if is_lover(weak_order):
-        return weak_order[0] + '>' + '~'.join(sorted([weak_order[2], weak_order[4]]))
-    else:
-        return '~'.join(sorted([weak_order[0], weak_order[2]])) + '>' + weak_order[4]
-
-
 def my_division(x, y):
     """
     Division of two numbers, trying to be exact if it is reasonable.
@@ -968,3 +698,76 @@ def my_division(x, y):
         return result.numerator
     else:
         return result
+
+
+def iterator_integers_fixed_sum(d, fixed_sum):
+    """Iterate over vectors of integers with a fixed sum.
+
+    Parameters
+    ----------
+    d : int
+        The desired number of integers. In other words, we consider a simplex of dimension `d - 1`.
+    fixed_sum : int
+        The fixed sum.
+
+    Yields
+    ------
+    tuple
+        Each tuple of `d` integers, whose sum is `fixed_sum`.
+
+    Examples
+    --------
+        >>> for t in iterator_integers_fixed_sum(d=3, fixed_sum=2):
+        ...     print(t)
+        (2, 0, 0)
+        (1, 1, 0)
+        (1, 0, 1)
+        (0, 2, 0)
+        (0, 1, 1)
+        (0, 0, 2)
+    """
+    if d == 1:
+        yield fixed_sum,
+    else:
+        for i in range(fixed_sum, -1, -1):
+            for r in iterator_integers_fixed_sum(d - 1, fixed_sum - i):
+                yield (i, *r)
+
+
+def iterate_simplex_grid(d, denominator):
+    """Iterate over the points in the simplex, with rational coordinates of a given denominator
+
+    Parameters
+    ----------
+    d : int
+        Number of coordinates. In other words, we consider the simplex of dimension `d - 1`.
+    denominator : int or iterable
+        The coordinates will be fractions with this denominator. If an iterable is given, we consider each
+        denominator given by the iterable.
+
+    Returns
+    -------
+    tuple
+        Each tuple of length `d`, whose coordinates are fractions of the given denominator, and whose sum is 1.
+
+    Examples
+    --------
+        >>> for t in iterate_simplex_grid(d=3, denominator=range(1, 3)):
+        ...     print(t)
+        (1, 0, 0)
+        (0, 1, 0)
+        (0, 0, 1)
+        (1, 0, 0)
+        (Fraction(1, 2), Fraction(1, 2), 0)
+        (Fraction(1, 2), 0, Fraction(1, 2))
+        (0, 1, 0)
+        (0, Fraction(1, 2), Fraction(1, 2))
+        (0, 0, 1)
+    """
+    if isinstance(denominator, int):
+        denominators = [denominator]
+    else:
+        denominators = denominator
+    for current_denominator in denominators:
+        for t in iterator_integers_fixed_sum(d, fixed_sum=current_denominator):
+            yield tuple(my_division(x, current_denominator) for x in t)
